package fr.imag.adele.maven.plugin.obr;


import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;

import bundles.obr.resource.BundleInfo;
import bundles.obr.resource.CapabilityImpl;
import bundles.obr.resource.RepositoryImpl;
import bundles.obr.resource.RequirementImpl;
import bundles.obr.resource.ResourceImpl;
import bundles.obr.resource.VersionImpl;

public class ExtractBindexInfo {

	private ResourceImpl m_resource;
	
	public ExtractBindexInfo(URI repoFilename, String outFile) throws MojoExecutionException{
		
		this.m_resource = null;
		RepositoryImpl repository = null;;
		try {
			repository = new RepositoryImpl(new File(repoFilename).getAbsoluteFile().toURL());
		} catch (MalformedURLException e) {
            e.printStackTrace();
            throw new MojoExecutionException("MalformedURLException");
		}
		BundleInfo info = null;
		try {
			info = new BundleInfo(repository, new File(outFile));
		} catch (Exception e) {
            e.printStackTrace();
            throw new MojoExecutionException("Exception");
		}
		
		try {
			m_resource = info.build();
		} catch (Exception e) {
            e.printStackTrace();
            throw new MojoExecutionException("Exception");
		}
	}
	
	private String parseFilter(String filter){
		filter.replaceAll("&", "&amp");
		filter.replaceAll(">=", "&gt");
		
		return filter;
	}
	public List getCapabilities(){
		ArrayList list = new ArrayList();
		Collection res = m_resource.getCapabilityList();
		Iterator it = res.iterator();
		while(it.hasNext())
		{
			Capability capability = new Capability();
			CapabilityImpl ci = ((CapabilityImpl)it.next());
			capability.setName(ci.getName());
			//System.out.println(ci.getName()) ;
			if(!(ci.getName().compareTo("bundle") == 0))
			{
				Map properties = ci.getProperties();
				for ( Iterator k= properties.keySet().iterator(); k.hasNext(); ) {
					PElement p = new PElement();
					String key = (String) k.next();
					List values = (List) properties.get(key);
					for ( Iterator v = values.iterator(); v.hasNext(); ) {
						Object value = v.next();
						p.setN(key);
						if ( value != null )
						{
							p.setV(value.toString());
						}
						else
							System.out.println("Missing value " + key);
						String type = null;
						if (value instanceof Number )
							type = "number";
						else if (value.getClass() == VersionImpl.class)
							type = "version";
						if (type != null)
						{
							p.setT(type);
						}
					}
					capability.addP(p);
				}
	
				list.add(capability);	
			}
		}
		return list; 
	}
	
/*	public Node addCapability(Node nodeCapabilities, Document parentDoc){
		Node node;
		if(nodeCapabilities == null)
		{
			node = parentDoc.createElement("capabilities");
		}
		else
			node = nodeCapabilities;
		
		Collection res = m_resource.getCapabilityList();
		Iterator it = res.iterator();
		while(it.hasNext())
		{
			Element capability = parentDoc.createElement("capability");
			CapabilityImpl ci = ((CapabilityImpl)it.next());
			capability.setAttribute("name", ci.getName());
			//System.out.println(ci.getName()) ;
			if(!(ci.getName().compareTo("bundle") == 0))
			{
				Map properties = ci.getProperties();
				for ( Iterator k= properties.keySet().iterator(); k.hasNext(); ) {
					Element p = parentDoc.createElement("p");
					String key = (String) k.next();
					List values = (List) properties.get(key);
					for ( Iterator v = values.iterator(); v.hasNext(); ) {
						Object value = v.next();
						p.setAttribute("n", key);
						if ( value != null )
						{
							p.setAttribute("v", value.toString());
						}
						else
							System.out.println("Missing value " + key);
						String type = null;
						if (value instanceof Number )
							type = "number";
						else if (value.getClass() == VersionImpl.class)
							type = "version";
						if (type != null)
						{
							p.setAttribute("t", type);
						}
					}
					capability.appendChild(p);
				}
	
				node.appendChild(capability);	
			}
		}
	
		return node;
	}*/
	public List getRequirement(){
		ArrayList list = new ArrayList();
		Collection res =  m_resource.getRequirementList();
		Iterator it = res.iterator();
		while(it.hasNext())
		{
			RequirementImpl ci = ((RequirementImpl)it.next());
			Require require = new Require();
			
			require.setExtend(String.valueOf(ci.isExtend()));
			require.setMultiple(String.valueOf(ci.isMultiple()));
			require.setOptional(String.valueOf(ci.isOptional()));
			require.setName(ci.getName());
			require.setFilter(this.parseFilter(ci.getFilter()));
			require.setValue(ci.getComment());
			list.add(require);
		}
		return list;
	}
/*	public Node addRequirement(Node nodeRequires, Document parentDoc){
		Node node;
		if(nodeRequires == null)
		{
			node = parentDoc.createElement("requires");
		}
		else
			node = nodeRequires;
		
		Collection res =  m_resource.getRequirementList();
		Iterator it = res.iterator();
		while(it.hasNext())
		{
			Element elem = parentDoc.createElement("require");
			RequirementImpl ci = ((RequirementImpl)it.next());
			elem.setAttribute("extend", String.valueOf(ci.isExtend()));
			elem.setAttribute("filter", this.parseFilter(ci.getFilter()));
			elem.setAttribute("multiple", String.valueOf(ci.isMultiple()));
			elem.setAttribute("name", ci.getName());
			elem.setAttribute("optional", String.valueOf(ci.isOptional()));
			elem.setTextContent(ci.getComment());
			node.appendChild(elem);
		}
		return node;
	}*/

	public String getSymbolicName(){
		return m_resource.getSymbolicName();
	}
	
	public String getVersion(){
		if(m_resource.getVersion() != null)
			return m_resource.getVersion().toString();
		else
			return null;
	}
	
	public String getPresentationName(){
		return m_resource.getPresentationName();
	}
	
	public String getCopyright(){
		return m_resource.getCopyright();
	}
	public String getDescription(){
		return m_resource.getDescription();
	}
	public String getDocumentation(){
		if(m_resource.getDocumentation() != null)
			return m_resource.getDocumentation().toString();
		else
			return null;
	}
	public String getLicense(){
		if(m_resource.getLicense() != null)
			return m_resource.getLicense().toString();
		else
			return null;
	}
	public String getSource(){
		if(m_resource.getSource() != null)
			return m_resource.getSource().toString();
		else
			return null;
	}
	
}
