package fr.imag.adele.maven.plugin.obr;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.maven.artifact.deployer.ArtifactDeployer;
import org.apache.maven.artifact.manager.WagonConfigurationException;
import org.apache.maven.artifact.manager.WagonManager;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.model.DistributionManagement;
import org.apache.maven.model.Resource;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.apache.maven.settings.Proxy;
import org.apache.maven.settings.Settings;
import org.apache.maven.wagon.CommandExecutionException;
import org.apache.maven.wagon.CommandExecutor;
import org.apache.maven.wagon.ConnectionException;
import org.apache.maven.wagon.ResourceDoesNotExistException;
import org.apache.maven.wagon.TransferFailedException;
import org.apache.maven.wagon.UnsupportedProtocolException;
import org.apache.maven.wagon.Wagon;
import org.apache.maven.wagon.authentication.AuthenticationException;
import org.apache.maven.wagon.authorization.AuthorizationException;
import org.apache.maven.wagon.observers.Debug;
import org.apache.maven.wagon.proxy.ProxyInfo;
import org.apache.maven.wagon.repository.Repository;
import org.codehaus.plexus.PlexusConstants;
import org.codehaus.plexus.PlexusContainer;
import org.codehaus.plexus.component.configurator.ComponentConfigurationException;
import org.codehaus.plexus.component.configurator.ComponentConfigurator;
import org.codehaus.plexus.component.repository.exception.ComponentLifecycleException;
import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.context.Context;
import org.codehaus.plexus.context.ContextException;


/**
* deploy the bundle to a ftp site
*
* @goal deployment
* @phase deploy
* @requiresDependencyResolution compile
*/

public class ObrDeploy extends AbstractMojo{

    private final String REPODEFAULTNAME = "repository.xml";
    /**
     * setting of maven.
     * @parameter expression="${settings}"
     * @require
     */

     private Settings settings;
     
     /**
      * name of the repository xml descriptor file
      *
      * @parameter alias="repository-name" default-value="repository.xml"
      */
     private String repositoryName;
     
    /**
     * The local Maven repository
     *
     * @parameter expression="${localRepository}" 
     * @required
     */
    private ArtifactRepository localRepo;

     
     /**
      * Project in use
      * @parameter expression="${project}"
      * @require
      */
      private MavenProject project;
      
      /**
       * @component
       */
      private WagonManager wagonManager;
      
      
      private String fileInLocalRepo;

      
	public void execute() throws MojoExecutionException, MojoFailureException {
        getLog().info("Obr-deploy start:");
        System.err.println(repositoryName);

        ArtifactRepository ar = project.getDistributionManagementArtifactRepository();
        
        //locate the obr.xml file
        String obrXmlFile = null;
        List l = project.getResources();
        for(int i=0;i<l.size();i++){
            File f = new File(((Resource)l.get(i)).getDirectory() + File.separator + "obr.xml");
            if(f.exists())
            {
                obrXmlFile = ((Resource)l.get(i)).getDirectory() + File.separator + "obr.xml";
                break;
            }
        }
        
        // the obr.xml file is not present
        if(obrXmlFile == null) 
        {
            getLog().warn("obr.xml is not present, use default");
        } 
        
        
        //create tempory file used to save the repository descriptor file
        File repoTempFile = null;
        try {
            repoTempFile = File.createTempFile("repoTemp", "tmp");
        } catch (IOException e1) {
            System.err.println("canno't create tempory file.");
            e1.printStackTrace();
            return;
        }
        
        File repoDescriptorFile = null;
        
        //get the repository descriptor xml file
        if (repositoryName == null) {
            repositoryName = REPODEFAULTNAME;
        }
        
        RemoteFileManager remoteFile = new RemoteFileManager(ar, wagonManager, settings,
                                                               getLog());
         
        remoteFile.connect();
        
        try {
            repoDescriptorFile = remoteFile.get(repositoryName);
        } catch (TransferFailedException e) {
            getLog().error("Transfer failed");
            e.printStackTrace();
            throw new MojoFailureException("TransferFailedException");
            
        } catch (ResourceDoesNotExistException e) {
            //file doesn't exist! create a new one
            getLog().warn("file specified does not exist: " + repositoryName);
            repositoryName = REPODEFAULTNAME;
            getLog().warn("Create a new repository descriptor file " + repositoryName);
            try {
                File f = File.createTempFile(String.valueOf(System.currentTimeMillis()), null);
                repoDescriptorFile = new File(f.getParent() + File.separator + String.valueOf(System.currentTimeMillis()) + ".xml");
            } catch (IOException e1) {
                getLog().error("canno't create temporary file");
                e1.printStackTrace();
                return;
            }
        } catch (AuthorizationException e) {
            getLog().error("Authorization failed");
            e.printStackTrace();
            throw new MojoFailureException("AuthorizationException");
        } catch (IOException e) {
            e.printStackTrace();
            throw new MojoFailureException("IOException");
        }
        
        Config userConfig = new Config();
        userConfig.setPathRelative(true);
        userConfig.setRemotely(true);
        
        PathFile file = null;
        
//      get the path to local maven repository
        file = new PathFile(PathFile.uniformSeparator(settings.getLocalRepository())+ 
            File.separator + PathFile.uniformSeparator(localRepo.pathOf(project.getArtifact())));
        if (file.isExists())
            fileInLocalRepo = file.getOnlyAbsoluteFilename();
        else
        {
            getLog().error("file not found in local repository: " + settings.getLocalRepository()+ File.separator + localRepo.pathOf(project.getArtifact()));
            return;
        }
        
        
        file = new PathFile("file:/"+repoDescriptorFile.getAbsolutePath());
        
        
        ObrUpdate obrUpdate = new ObrUpdate(file, obrXmlFile, project, fileInLocalRepo,PathFile.uniformSeparator(settings.getLocalRepository()),
                                            userConfig, getLog());
        
        obrUpdate.UpdateRepository();
        
        //the reposiroty descriptor file is modified, we upload it on the remote repository
        try {
            remoteFile.put(repoDescriptorFile, repositoryName);
        } catch (TransferFailedException e) {
            getLog().error("Transfer failed");
            e.printStackTrace();
            throw new MojoFailureException("TransferFailedException");
        } catch (ResourceDoesNotExistException e) {
            getLog().error("Resource does not exist:" + repoDescriptorFile.getName());
            e.printStackTrace();
            throw new MojoFailureException("ResourceDoesNotExistException");
        } catch (AuthorizationException e) {
            getLog().error("Authorization failed");
            e.printStackTrace();
            throw new MojoFailureException("AuthorizationException");
        }
        repoDescriptorFile.delete();
        remoteFile.disconnect();
    }
}
