package fr.imag.adele.maven.plugin.obr;

import java.io.File;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.settings.Settings;


/**
 * @description construct the repository.xml from a compiled bundle
 * 
 * @goal install-file
 * @requiresProject false
 * @phase install
 * @requiresDependencyResolution runtime
 */
public class ObrInstallFile extends AbstractMojo{
	/**
	 * The local Maven repository
	 *
	 * @parameter expression="${localRepository}"
	 * @required
	 */
	private ArtifactRepository localRepo;
    /**
     * path to the repository.xml
     *
     * @parameter alias="repository-path"
     * @require
     */
    private String repositoryPath;

    /**
     * setting of maven.
     * @parameter expression="${settings}"
     * @require
     */
     private Settings settings;
	
    /**
     * @descritpion symbolic name define by the user
     * @parameter expression="${artifactId}"
     */
	private String artifactId;
	
    /**
     * @descritpion groupId define by the user
     * @parameter expression="${groupId}"
     */
	private String groupId;
	
    /**
     * @descritpion version define by the user
     * @parameter expression="${version}"
     */
	private String version;
	
    /**
     * @descritpion packaging define by the user
     * @parameter expression="${packaging}"
     */
	private String packaging;
	
    /**
     * @descritpion obr file define by the user
     * @parameter expression="${obrFile}"
     */
	private String obrFile;
	
    /**
     * Project in use
     * @parameter expression="${project}"
     * @require
    */

    private MavenProject project;
	
	public void execute() throws MojoExecutionException, MojoFailureException {
		getLog().info("Install-File Obr starts:");

		PathFile fileOut;
		
		if(groupId == null){
			getLog().error("-DgroupId=VALUE is required");
			return;
		}
		if(artifactId == null){
			getLog().error("-Dartifactid=VALUE is required");
			return;
		}
		if(version == null){
			getLog().error("-Dversion=VALUE is required");
			return;
		}
		if(packaging == null){
			getLog().error("-Dpackaging=VALUE is required");
			return;
		}
		//UserProperties  user = new UserProperties(artifactId, groupId, version);
		
		
		//copy the file to the local repository
		PathFile repoLocal = new PathFile(localRepo.getBasedir());
		
        
        //get the target file in mvn repo
		fileOut = new PathFile(PathFile.uniformSeparator(settings.getLocalRepository())+ 
                File.separator + PathFile.uniformSeparator(localRepo.pathOf(project.getArtifact())));//pathOut+filenameOut);
		
		if(!fileOut.isExists()){
			getLog().error("file doesn't exist: " + fileOut.getAbsoluteFilename());
			return;
		}
		else
			getLog().info("Target file: " + fileOut.getAbsoluteFilename());
		
		
    	if(repositoryPath == null){
    		repositoryPath = "file:" + repoLocal.getOnlyAbsoluteFilename() + "repository.xml";
    		getLog().warn("-DpathRepo is not define, use default repository: " + repositoryPath);
    	}
    	
    	PathFile fileRepo = new PathFile(repositoryPath);
    	if(fileRepo.isRelative())
    		fileRepo.setBaseDir(settings.getLocalRepository());
    	
    	//create the folder to the repository
    	PathFile repoExist = new PathFile(fileRepo.getAbsolutePath());
    	if(!repoExist.isExists())
    		fileRepo.createPath();
    	 

    	PathFile fileObrXml = new PathFile(obrFile);
	    if(!fileObrXml.isExists()){
	    	getLog().warn("obr.xml file not found, use default");
	    }
    	
	    //build the user config
	    Config userConfig = new Config();
	    
        /*System.out.println(project.getArtifactId());
        System.out.println(project.getGroupId());
        System.out.println(project.getVersion());*/
	    
    	ObrUpdate obrUpdate = new ObrUpdate(fileRepo, fileObrXml.getOnlyAbsoluteFilename(), project,
    			fileOut.getOnlyAbsoluteFilename(), localRepo.getBasedir(), userConfig, getLog());
    	obrUpdate.UpdateRepository();
		
		
	}

}
