package fr.imag.adele.maven.plugin.obr;


import java.io.File;
import java.util.List;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.model.Resource;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.settings.Settings;

/**
 * construct the repository.xml with a project Maven compiled
 *
 * @goal repository
 * 
 * @phase install
 * @requiresDependencyResolution compile
 */
public class ObrPlugin extends AbstractMojo{
	/**
	 * The local Maven repository
	 *
	 * @parameter expression="${localRepository}"
	 * @required
	 */
	private ArtifactRepository localRepo;
    /**
     * path to the repository.xml
     *
     * @parameter alias="repository-path" 
     * @require
     */
    private String repositoryPath;
    
    /**
     * path to the project
     *
     * @parameter expression="${project.build.directory}"
     * @require
     */
    private String pathOutput;
    
    /**
     * final name
     *
     * @parameter expression="${project.build.finalName}"
     * @require
     */
    private String finalName;
    
    /**
    * Project in use
    * @parameter expression="${project}"
    * @require
    */

    private MavenProject project;

    /**
     * setting of maven.
     * @parameter expression="${settings}"
     * @require
     */

     private Settings settings;
     
     
    /**
     * path to file in the maven local repository
     */
	private String fileInLocalRepo;
     
    public void execute() throws MojoExecutionException{
    	getLog().info("Obr Plugin starts:");
 
    	if(repositoryPath == null){
    		repositoryPath = "file:/" + localRepo.getBasedir() + File.separator + "repository.xml";
    		getLog().warn("-DpathRepo is not define, use default repository: " + repositoryPath);
    	}
    		
    	
    	PathFile file = new PathFile(repositoryPath);
    	if(file.isExists()){
    		if(!repositoryPath.startsWith("file:/"))
    			repositoryPath = "file:/" + repositoryPath;
    	}

    	//locate the obr.xml file
    	String obrXmlFile = null;
    	List l = project.getResources();
    	for(int i=0;i<l.size();i++){
    		File f = new File(((Resource)l.get(i)).getDirectory() + File.separator + "obr.xml");
    		if(f.exists())
    		{
    			obrXmlFile = ((Resource)l.get(i)).getDirectory() + File.separator + "obr.xml";
    			break;
    		}
    	}
    	// the obr.xml file is not present
    	if(obrXmlFile == null) 
    	{
    		getLog().warn("obr.xml is not present, use default");
    	} 
    	
    	//get the path to local maven repository
    	 file = new PathFile(PathFile.uniformSeparator(settings.getLocalRepository())+ 
    			File.separator + PathFile.uniformSeparator(localRepo.pathOf(project.getArtifact())));
    	if (file.isExists())
    		fileInLocalRepo = file.getOnlyAbsoluteFilename();
    	else
    	{
    		getLog().error("file not found in local repository: " + settings.getLocalRepository()+ File.separator + localRepo.pathOf(project.getArtifact()));
    		return;
    	}
    	
    	//verify the repository.xml
    	PathFile fileRepo = new PathFile(repositoryPath);
    	if(fileRepo.isRelative())
    		fileRepo.setBaseDir(settings.getLocalRepository());

    	//create the folder to the repository
    	PathFile repoExist = new PathFile(fileRepo.getAbsolutePath());
    	if(!repoExist.isExists())
    		fileRepo.createPath();
    	
    	
    	//build the user configuration (use default)
    	Config user = new Config();
    	
    	
    	//System.err.println(outputFile.getOnlyAbsoluteFilename());
    	ObrUpdate obrUpdate = new ObrUpdate(fileRepo, obrXmlFile, project, fileInLocalRepo, PathFile.uniformSeparator(settings.getLocalRepository()) ,
    										user, getLog());
    	try {
            obrUpdate.UpdateRepository();
        } catch (MojoExecutionException e) {
            e.printStackTrace();
            throw new MojoExecutionException("MojoFailureException");
        }
    }
    
}
