
package fr.imag.adele.maven.plugin.obr;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


 

public class ObrUpdate {

	private Log m_logger;
	
	private URI m_repoFilename; 		//name and path to the repository.xml file
	private String m_obrXml;			//name and path to the obr.xml file
	private String m_localRepo;			//path to the maven local repository
	private String m_outputFile;		//name and path to the bundle jar file
	private MavenProject m_project;		//maven project
	//private UserProperties m_userProps;		//informations given by the user

	private Config m_userConfig;
	
	private DocumentBuilder m_constructor;
	

	private Document m_repoDoc;			//m_root document parent

	private boolean[] m_idTab;			//use to determine the first id free
	
	private Node m_root;				//node root to repertory tree
	
	private ExtractBindexInfo m_ebi;	//calcul Bindex bundle informations
	 
	private ResourcesBundle m_resourceBundle;	//class used to store Bundle informations
	
	//private String m_ext;				//reporitory.xml file tempory extension
	
	static SimpleDateFormat	format		= new SimpleDateFormat("yyyyMMddHHmmss.SSS");
	
	
	//test
	private PathFile m_repo;			//pathfile to the repository.xml
	
	
	
	public ObrUpdate(PathFile repoFilename, String obrXml, MavenProject project, String outputFile,
						String localRepo, Config userConfig, Log log) {	
		this.m_localRepo = localRepo;
		this.m_outputFile = outputFile;
		this.m_repoFilename = repoFilename.getUri();
		this.m_obrXml = obrXml;
		this.m_project = project;//
		this.m_logger = log;

		this.m_userConfig = userConfig;
		//init the tab
		m_idTab = new boolean[0];

		m_resourceBundle = new ResourcesBundle(log);
	
        if(userConfig.isRemotely()) {
            this.m_repo = new PathFile(localRepo);
        } else {	
            this.m_repo = repoFilename;
        }
        //System.err.println("Construct: "+repoFilename.getAbsoluteFilename());
	}
	

	public void UpdateRepository() throws MojoExecutionException{

		m_constructor = initConstructor();
		
		if(m_constructor == null)
			return;
		
		//get the file size
		PathFile pf = new PathFile(m_outputFile);
		File fout = pf.getFile();
		pf.setBaseDir(m_repo.getOnlyAbsolutePath());
		if(fout.exists())
		{
			m_resourceBundle.setSize(String.valueOf(fout.length()));
			if(m_userConfig.isPathRelative())
				m_resourceBundle.setUri(pf.getOnlyRelativeFilename().replace('\\', '/'));
			else
				m_resourceBundle.setUri("file:"+m_outputFile);
		}
		else
		{
			m_logger.error("file doesn't exist: " + m_outputFile);
			return;
		}
		
		//parse repository
		if(parseRepositoryXml() == -1)
		{
			return;
		}
		
		
		//parse the obr.xml file
		if(m_obrXml != null){
			URL url = getClass().getResource("/SchemaObr.xsd");
			//TODO validate obr.xml file
			
			Document obrXmlDoc = parseFile(m_obrXml, m_constructor);
			if(obrXmlDoc == null)
				return;
			Node obrXmlRoot = (Element) obrXmlDoc.getDocumentElement();
			// sort the obr file
			sortObrXml(obrXmlRoot);
		}
		
		//use bindex to extract bundle information
		try {
            m_ebi = new ExtractBindexInfo(m_repoFilename, m_outputFile);
        } catch (MojoExecutionException e) {
            m_logger.error("unable to build Bindex informations");
            e.printStackTrace();
            throw new MojoExecutionException("MojoFailureException");
        }

		m_resourceBundle.construct(m_project,m_ebi);
		
		
		
		if(!walkOnTree(m_root))
		{
			//the correct resource node was not found, we must create it
			//we calcul the new id
			int id = -1;
			for(int i=1;i<m_idTab.length;i++)
			{
				if(!m_idTab[i])
				{
					id = i;
					break;
				}
			}
			if(id == -1)
				id = m_idTab.length;

			searchRepository(m_root, id);
		}
		
		
		//the repository.xml file have been modified, so we save it
		writeToFile(m_repoFilename, m_root);
	}
	
	
	private DocumentBuilder initConstructor() {
		DocumentBuilder constructor = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			constructor = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			m_logger.error("unable to create a new xml document");
			e.printStackTrace();
		}
		return constructor;
		
	}
	private int parseRepositoryXml() throws MojoExecutionException{
		
		File fout = new File(m_repoFilename.getPath());
		if(!fout.exists())
		{
			//create the repository.xml
			try {
				fout.createNewFile();
				m_logger.info("create new repository.xml file");
			} catch (IOException e) {
				m_logger.error("cannot create "+m_repoFilename.getPath());
				e.printStackTrace();
				return -1;
			}

			Document doc = m_constructor.newDocument();
			//create xml tree
			Date d = new Date();
			d.setTime(System.currentTimeMillis());
			Element root = doc.createElement("repository");
			root.setAttribute("lastmodified", format.format(d));
			root.setAttribute("name", "MyRepository");
			try {
                writeToFile(m_repoFilename, root);
            } catch (MojoExecutionException e) {
                e.printStackTrace();
                throw new MojoExecutionException("MojoExecutionException");
            }
		}
		
		//now we parse the repository.xml file
		m_repoDoc = parseFile(m_repoFilename.getPath()/*+m_ext*/, m_constructor);
		if(m_repoDoc == null){
			return -1;
		}
		
		m_root = (Element)m_repoDoc.getDocumentElement();
		return 0;
		
	}
	
	private Document parseFile(String filename, DocumentBuilder constructor){
		if(constructor == null)
			return null;
		// The document is the root of the DOM tree.
		m_logger.info("try to open: "+filename);
		Document doc = null;
		try {
			doc = constructor.parse(new File(filename));
		} catch (SAXException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			m_logger.error("cannot open file: " +filename);
			e.printStackTrace();
			return null;
		}
		return doc;
	}

	private void sortObrXml(Node node){
		if(node.getNodeName().compareTo("require") == 0){
			Require newRequireNode = new Require();
			NamedNodeMap list = node.getAttributes();
			try{
				newRequireNode.setExtend(list.getNamedItem("extend").getNodeValue());
				newRequireNode.setMultiple(list.getNamedItem("multiple").getNodeValue());
				newRequireNode.setOptional(list.getNamedItem("optional").getNodeValue());
				newRequireNode.setFilter(list.getNamedItem("filter").getNodeValue());
				newRequireNode.setName(list.getNamedItem("name").getNodeValue());
			}
			catch(NullPointerException e){
				m_logger.error("the obr.xml file seems to be invalid in a \"require\" tag (one or more attributes are missing)");
				//e.printStackTrace();
			}
			newRequireNode.setValue(node.getTextContent());
			m_resourceBundle.addRequire(newRequireNode);
		}
		else if(node.getNodeName().compareTo("capability") == 0){
			Capability newCapability = new Capability();
			try{
			newCapability.setName(node.getAttributes().getNamedItem("name").getNodeValue());
			}catch(NullPointerException e){
				m_logger.error("attribute \"name\" is missing in obr.xml in a \"capability\" tag");
				e.printStackTrace();
			}
			NodeList list = node.getChildNodes();
			for(int i=0;i<list.getLength();i++){
				PElement p = new PElement();
				Node n = list.item(i);
				Node item = null;
				//System.err.println(n.getNodeName());
				if(n.getNodeName().compareTo("p")==0){
					
					p.setN(n.getAttributes().getNamedItem("n").getNodeValue());
					item = n.getAttributes().getNamedItem("t");
					if(item != null)
						p.setT(item.getNodeValue());
					item = n.getAttributes().getNamedItem("v");
					if(item != null)
						p.setV(item.getNodeValue());
					
					newCapability.addP(p);
				}
			}
			m_resourceBundle.addCapability(newCapability);
		}
		else if(node.getNodeName().compareTo("category") == 0){
			Category newCategory = new Category();
			newCategory.setId(node.getAttributes().getNamedItem("id").getNodeValue());
			m_resourceBundle.addCategory(newCategory);
		}
		else {
			NodeList list = node.getChildNodes();
			for(int i=0;i<list.getLength();i++)
			{
				sortObrXml(list.item(i));
			}
		}
	}
	private void writeToFile(URI outputFilename, Node treeToBeWrite) throws MojoExecutionException{
		//init the transformer
		Transformer transformer = null;
		TransformerFactory tfabrique = TransformerFactory.newInstance();
		try {
			transformer = tfabrique.newTransformer();
		} catch (TransformerConfigurationException e) {
            m_logger.error("Unable to write to file: " + outputFilename.toString());
            e.printStackTrace();
            throw new MojoExecutionException("TransformerConfigurationException");
		}
		Properties proprietes = new Properties();
		proprietes.put("method", "xml");
		proprietes.put("version", "1.0"); 
		proprietes.put("encoding", "ISO-8859-1"); 
		proprietes.put("standalone", "yes");
		proprietes.put("indent", "yes");
		proprietes.put("omit-xml-declaration", "no");
		transformer.setOutputProperties(proprietes);
		
		DOMSource input = new DOMSource(treeToBeWrite);
		
        File fichier = new File(outputFilename);
		FileOutputStream flux = null;
		try {
			flux = new FileOutputStream(fichier);
		} catch (FileNotFoundException e) {
            m_logger.error("Unable to write to file: " + fichier.getName());
            e.printStackTrace();
            throw new MojoExecutionException("FileNotFoundException");
		}
		Result output = new StreamResult(flux);
		try {
			transformer.transform(input, output);
		} catch (TransformerException e) {
            e.printStackTrace();
            throw new MojoExecutionException("TransformerException");
		}
		
		try {
			flux.flush();
            flux.close();
		} catch (IOException e) {
            e.printStackTrace();
            throw new MojoExecutionException("IOException");
		}

	}
	
	private boolean walkOnTree(Node node)
	{

		if(node.getNodeName().compareTo("resource") == 0)
		{
			if(resource(node))
				return true;
			else
				return false;
		} 
		else
		{
			if(node.getNodeName().compareTo("repository") == 0)
			{
				Date d = new Date();
				d.setTime(System.currentTimeMillis());
				NamedNodeMap nList =  node.getAttributes();
				Node n = nList.getNamedItem("lastmodified");
				n.setNodeValue(format.format(d));
			}
			NodeList list = node.getChildNodes();
			if(list.getLength() > 0)
			{
				for(int i=0;i<list.getLength();i++)
				{
					if(walkOnTree(list.item(i)))
						return true;
				}
			}
			return false;
		}
		
	}

	private void searchRepository(Node node, int id)
	{
		if(node.getNodeName().compareTo("repository") == 0)
		{
			m_resourceBundle.setId(String.valueOf(id));
			node.appendChild(m_resourceBundle.getNode(m_repoDoc));
			return;
		}
		else
		{
			NodeList list = node.getChildNodes();
			if(list.getLength() > 0)
			{
				for(int i=0;i<list.getLength();i++)
				{
					searchRepository(list.item(i), id);
				}
			}
		}
			
	}

	private boolean resource(Node node) {
		
		//this part save all the id free if we need to add resource
		int id = Integer.parseInt(node.getAttributes().getNamedItem("id").getNodeValue());
		if(id>=m_idTab.length)
		{
			//resize tab
			boolean[] bt = new boolean[id+1];
			for(int i=0;i<id+1;i++)
				if(m_idTab.length>i && m_idTab[i])
					bt[i] = true;
				else
					bt[i] = false;
			
			m_idTab = bt;
		}
		m_idTab[id] = true;
		
		
		NamedNodeMap map = node.getAttributes();
		
		if(m_resourceBundle.isSameBundleResource(map.getNamedItem("symbolicname").getNodeValue(),
					map.getNamedItem("presentationname").getNodeValue(), 
					map.getNamedItem("version").getNodeValue()))
		{
			m_resourceBundle.setId(String.valueOf(id));
			node.getParentNode().replaceChild(m_resourceBundle.getNode(m_repoDoc), node);
			return true;
		}
		return false;
	}
}
