package fr.imag.adele.maven.plugin.obr;
 
import java.io.File; 
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.channels.FileChannel;

public class PathFile {
	
	private String m_fullFilename;
	private String m_fileName;
	private String m_pathFile;
	private String m_baseDir;
	private String m_protocol;
	private boolean m_relative;
	private boolean m_folder;
	private boolean m_exist;
	private boolean m_file;
	private boolean m_valid;
	
	
	public PathFile(String filename){

			
		this.m_fullFilename = filename;
		if(filename == null)
		{
			this.m_valid = false;
			return;
		}
		this.m_valid = true;
		m_protocol = extractProtocol(filename);
		m_pathFile = extractPathFile(filename);
		if(m_pathFile.startsWith("//")){
			//avoid problems on Unix like system
			m_pathFile = m_pathFile.substring(1);
		}
		m_fileName = extractFileName(filename);
		m_relative = extractRelative();
		if(!m_relative && (getProtocol().compareTo("file")==0 || getProtocol().compareTo("")==0)){
			File f = new File(getOnlyAbsoluteFilename());
			m_file = f.isFile();
			m_folder = f.isDirectory();
			m_exist = f.exists();
            if(m_folder) {
                m_pathFile = m_pathFile + m_fileName + File.separator;
                m_fileName = "";
            }
		}
		if(m_exist){
			m_protocol = "file";
		}
		else{
			if(m_fileName.compareTo("")==0){
				m_folder = true;
				m_file = false;
			}
			else
			{
				m_folder = false;
				m_file = true;
			}

		}
		
		//add the '/' before the complete path if it is absolute path
		if(!this.isRelative() && !m_pathFile.startsWith("/"))
			m_pathFile = "/".concat(m_pathFile);
		
	}
	private boolean extractRelative(){
		if(m_pathFile.startsWith("."+File.separator,1) || m_pathFile.startsWith(".."+File.separator,1)){
			m_pathFile = m_pathFile.substring(1);
			m_valid = false;
			return true;
		}
		
		
		return false;
	}
	
	private String extractFileName(String fullFilename){
		int index = fullFilename.lastIndexOf(File.separator);
		return fullFilename.substring(index+1, fullFilename.length());
	}
	private String extractPathFile(String fullFilename){
		String substring;
		if(extractFileName(fullFilename).compareTo("")==0){
			//it is a folder
			substring = fullFilename;
		}
		else
			substring = fullFilename.substring(0, fullFilename.indexOf(extractFileName(fullFilename)));
		
		if(getProtocol().compareTo("")!=0){
			substring = substring.substring(5);
		}

		return substring;
	}
	
	private String extractProtocol(String filename){
		if(filename.startsWith("file:")){
			return "file";
		}
		if(filename.startsWith("http:")){
			return "http";
		}
		return "";
	}
	public void setBaseDir(String baseDir){
		this.m_baseDir = baseDir;
		if(isRelative() && this.m_fullFilename != null){
			this.m_valid = true;
			if((getProtocol().compareTo("file")==0 || getProtocol().compareTo("")==0)){
				File f = new File(getOnlyAbsoluteFilename());
				m_file = f.isFile();
				m_folder = f.isDirectory();
				m_exist = f.exists();
			}
			if(m_exist){
				m_protocol = "file";
			}
		}
		
	}
	public String getBaseDir(){
		return this.m_baseDir;
	}
	public boolean isValid(){
		return m_valid;
	}
	public boolean isRelative(){
		return m_relative;
	}
	public boolean isExists(){
		return m_exist;
	}
	public boolean isFile(){
		return m_file;
	}
	public boolean isFolder(){
		return m_folder;
	}
	public File getFile(){
		if(!this.isValid()){
			return null;
		}
		String path = PathFile.uniformSeparator(this.getOnlyAbsoluteFilename());
		if(File.separatorChar == '\\')
			path = path.replace('\\', '/');
		File f = new File(path);
		return f;
	}
	
	public URI getUri(){
		if(!this.isValid())
			return null;
		String path = PathFile.uniformSeparator( getAbsoluteFilename() );
		if(File.separatorChar == '\\')
			path = path.replace("\\", "/");
		
		URI uri = null;
		try {
			 uri = new URI(path);
		} catch (URISyntaxException e) {
			System.err.println("malformed uri: "+path);
			return null;
		}
		return uri;
	}
	
	public String getRelativePath(){
		if(!this.isValid())
			return null;
		
		return getProtocol() + ":/" + getOnlyRelativePath();
	}
	
	public String getOnlyRelativePath(){
		if(!this.isValid())
			return null;
		if(this.isRelative()){
			return m_pathFile;
			
		}
		else
		{
			if(m_baseDir != null){
				//System.err.println(m_pathFile);
				//System.err.println(m_baseDir);
				if(m_pathFile.startsWith(m_baseDir)){
					/*String ch1 = m_pathFile;
					String ch2 = m_baseDir;
					System.err.println(ch1);
					System.err.println(ch2);
					System.err.println("."+File.separator+ch1.substring(ch2.length()));*/
					return "." + File.separator + m_pathFile.substring(m_baseDir.length());
				}
			}
			return null;
		}
	}
	
	private String calculAbsolutePath(String baseDir, String path){
		if(path.startsWith(".."+File.separatorChar)){
			String base = baseDir;
			int lastIndex;
			lastIndex = base.lastIndexOf(File.separator);
			if(lastIndex == base.length()){
				base = base.substring(0, base.length()-1);
				lastIndex = base.lastIndexOf(File.separator);
			}
			if(lastIndex<base.length()){
				return calculAbsolutePath(base.substring(0, lastIndex+1), path.substring(3));
			}
			else
				return null;
		}
		else if(path.startsWith("."+File.separatorChar)){
			String res;
			if(File.separatorChar == '\\')
				res = path.replaceFirst(".", baseDir.replace('\\', '/'));
			else
				res = path.replaceFirst(".", baseDir);
			
			return PathFile.uniformSeparator(res);
		}
		else
			return PathFile.uniformSeparator(baseDir + path );
	}

	public String getOnlyAbsolutePath(){
		if(!this.isValid())
			return null;
		if(isRelative()){
			return calculAbsolutePath( m_baseDir, m_pathFile);
		}
		else
			return m_pathFile;
	}
	
	public String getAbsolutePath(){
		
		if(isRelative()){
			return getProtocol() + ":/" + calculAbsolutePath( m_baseDir, m_pathFile);
		}
		else
			
			if(getProtocol().compareTo("")==0 || m_pathFile == null)
				return m_pathFile;
			else
				return getProtocol() + ":" +m_pathFile;
	}
	
	public String getOnlyAbsoluteFilename(){
		if(getOnlyAbsolutePath() != null && getFilename() != null)
			return getOnlyAbsolutePath() + getFilename();
		else
			return null;
	}
	
	public String getAbsoluteFilename(){
		if(getAbsolutePath() != null && getFilename() != null)
			return getAbsolutePath() + getFilename();
		else
			return null;
	}
	
	public String getOnlyRelativeFilename(){

		if(!this.isValid())
			return "";
		
		return getOnlyRelativePath() + getFilename();

	}
	
	public String getRelativeFilename(){
		if(!this.isValid())
			return "";
		
		if(this.isRelative())
			return getRelativePath() + getFilename();
		else
			return getAbsoluteFilename();
	}
	
	public String getFilename(){
		return m_fileName;
	}

	public String getProtocol(){
		return m_protocol;
	}
	
	public boolean createPath(){
		File path = new File(this.getOnlyAbsolutePath());
		if(path.exists())
			return true;
		return path.mkdirs();
	}
	public boolean createFile(){
		File path = new File(this.getOnlyAbsolutePath());
		if(!path.exists()){
			if(!this.createPath())
				return false;
		}
		path = new File(this.getOnlyAbsoluteFilename());
		try {
			return path.createNewFile();
		} catch (IOException e) {
			return false;
		}
		
	}
	public boolean delete() {
		File path = new File(this.getAbsoluteFilename());
		if(path.exists()){
			return path.delete();
		}
		else 
			return true;
		
	}
	public static String doubleSeparator(String path){
		//double the '\' in the path
		if(path != null && File.separatorChar == '\\')
			return path.replace("\\", "\\\\");
		else
			return null;
	}
	
	public static String uniformSeparator(String path){
		if(File.separatorChar == '\\'){
			if(path.startsWith("/"))
				return path.substring(1).replace('/', File.separatorChar);
			else
				return path.replace('/', File.separatorChar);
		}
		else
		{
			return path.replace('\\', File.separatorChar); 
		}
	}
	public static boolean copyFile(PathFile src, PathFile dest){
		FileChannel in = null; 
		FileChannel out = null; 
		
		if(!src.isExists()){
			System.err.println("src file must exist: "+src.getAbsoluteFilename());
			return false;
		}
		if(!dest.isExists()){
			dest.createFile();
		}
		try {
		  in = new FileInputStream(src.getOnlyAbsoluteFilename()).getChannel();
		  out = new FileOutputStream(dest.getOnlyAbsoluteFilename()).getChannel();
		 
		  in.transferTo(0, in.size(), out);
		} catch (Exception e) {
		  e.printStackTrace(); 
		} finally { 
		  if(in != null) {
		  	try {
			  in.close();
			} catch (IOException e) {return false;}
		  }
		  if(out != null) {
		  	try {
			  out.close();
			} catch (IOException e) {return false;}
		  }
		}
		return true;
	}

	

}
