package fr.imag.adele.maven.plugin.obr;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/*
 * this class contains the resource bundle information
 */

public class ResourcesBundle {
	private String m_symbolicName;
	private String m_presentationName;
	private String m_version;
	private String m_uri;
	private String m_description;
	private String m_size;
	private String m_documentation;
	private String m_source;
	private String m_license;
	private String m_id;
	private List m_category = new ArrayList();
	private List m_capability = new ArrayList();
	private List m_require	=	new ArrayList();
	
	private Log m_logger;
	
	public ResourcesBundle(Log log){
		m_logger = log;
	}
	
	public List getCapability() {
		return m_capability;
	}
	public void setCapability(List capability) {
		this.m_capability = capability;
	}
	public List getCategory() {
		return m_category;
	}
	public void setCategory(List category) {
		this.m_category = category;
	}
	public String getLicense() {
		return m_license;
	}
	public void setLicense(String license) {
		this.m_license = license;
	}
	public String getDescription() {
		return m_description;
	}
	public void setDescription(String m_description) {
		this.m_description = m_description;
	}
	public String getDocumentation() {
		return m_documentation;
	}
	public void setDocumentation(String m_documentation) {
		this.m_documentation = m_documentation;
	}
	public String getPresentationName() {
		return m_presentationName;
	}
	public void setPresentationName(String name) {
		m_presentationName = name;
	}
	public String getSize() {
		return m_size;
	}
	public void setSize(String m_size) {
		this.m_size = m_size;
	}
	public String getSymbolicName() {
		return m_symbolicName;
	}
	public void setSymbolicName(String name) {
		m_symbolicName = name;
	}
	public String getUri() {
		return m_uri;
	}
	public void setUri(String m_url) {
		this.m_uri = m_url;
	}
	public String getVersion() {
		return m_version;
	}
	public void setVersion(String m_version) {
		this.m_version = m_version;
	}
	public List getRequire() {
		return m_require;
	}
	public void setRequire(List require) {
		this.m_require = require;
	}
	public String getSource() {
		return m_source;
	}
	public void setSource(String source) {
		this.m_source = source;
	}
	public void addCapability(Capability capability){
		m_capability.add(capability);
	}
	public void addRequire(Require require){
		m_require.add(require);
	}
	public void addCategory(Category category){
		m_category.add(category);
	}
	
	
	
	private List getNodeCategories(Document father){
		List listNode = new ArrayList();
		List listCategory = (ArrayList) this.getCategory();
		for(int i=0;i<listCategory.size();i++){
			listNode.add( ((Category) listCategory.get(i)).getNode(father));
		}
		return listNode;
	}
	private List getNodeCapabilities(Document father){
		List listNode = new ArrayList();
		List listCapability = (ArrayList) this.getCapability();
		for(int i=0;i<listCapability.size();i++){
			listNode.add(  ((Capability) listCapability.get(i)).getNode(father));
		}
		return listNode;
	}
	private List getNodeRequirement(Document father){
		List listNode = new ArrayList();
		List listRequirement = (ArrayList) this.getRequire();
		for(int i=0;i<listRequirement.size();i++){
			listNode.add(  ((Require) listRequirement.get(i)).getNode(father));
		}
		return listNode;
	}
	
	 
	
	
	public Node getNode(Document father){
		//return the complete resource tree 
		if(!this.isValid() || this.getId() == null){
			m_logger.error("those properties was not defined:"+this.getInvalidProperties());
			return null;
		}
		
		Element resource 		= father.createElement("resource");
		Element description 	= father.createElement("description");
		Element size 			= father.createElement("size");
		Element documentation	= father.createElement("documentation");
		Element source			= father.createElement("source");
		Element license			= father.createElement("license");
		
		resource.setAttribute("id", this.getId());
		resource.setAttribute("symbolicname", this.getSymbolicName());
		resource.setAttribute("presentationname", this.getPresentationName());
		resource.setAttribute("uri", this.getUri());
		resource.setAttribute("version", this.getVersion());
		
		description.setTextContent(this.getDescription());
		resource.appendChild(description);
		
		size.setTextContent(this.getSize());
		resource.appendChild(size);
		
		documentation.setTextContent(this.getDocumentation());
		resource.appendChild(documentation);
		
		source.setTextContent(this.getSource());
		resource.appendChild(source);
		
		license.setTextContent(this.getLicense());
		resource.appendChild(license);
		
		ArrayList list = (ArrayList) this.getNodeCategories(father);
		for(int i=0;i<list.size();i++){
			resource.appendChild((Node)list.get(i));
		}
		
		list = (ArrayList) this.getNodeCapabilities(father);
		for(int i=0;i<list.size();i++){
			resource.appendChild((Node)list.get(i));
		}
		
		list = (ArrayList) this.getNodeRequirement(father);
		for(int i=0;i<list.size();i++){
			resource.appendChild((Node)list.get(i));
		}
		
		return resource;
	}

	public boolean isEqual(){
		return true;
	}
	/*
	 * methode used to build resourcebundle with only jar file (without compiled by maven)
	 */
	public boolean construct(UserProperties user, ExtractBindexInfo ebi){

		this.setSymbolicName(user.getArtefactId());
		this.m_logger.warn("user defined Symbolic name overrides manifest symbolic name");
		this.setPresentationName(ebi.getPresentationName());
		//this.setVersion(ebi.getVersion());
		this.setVersion(user.getVersion());
		this.m_logger.warn("user defined Version overrides manifest version");
		
		this.setDescription(ebi.getDescription());
		this.setDocumentation(ebi.getDocumentation());
		this.setSource(ebi.getSource());
		this.setLicense(ebi.getLicense());

		
		//create the first capability (ie : bundle)
		Capability capability = new Capability();
		capability.setName("bundle");
		PElement p = new PElement();
		p.setN("manifestversion");
		p.setV("2");
		capability.addP(p);
		
		p = new PElement();
		p.setN("presentationname");
		p.setV(this.getPresentationName());
		capability.addP(p);
		
		p = new PElement();
		p.setN("symbolicname");
		p.setV(this.getSymbolicName());
		capability.addP(p);
		
		p = new PElement();
		p.setN("version");
		p.setT("version");
		p.setV(this.getVersion());
		capability.addP(p);
		
		this.addCapability(capability);
		
		ArrayList capabilities = (ArrayList) ebi.getCapabilities();
		for(int i=0;i<capabilities.size();i++){
			this.addCapability((Capability)capabilities.get(i));
		}
		
		ArrayList requirement = (ArrayList) ebi.getRequirement();
		for(int i=0;i<requirement.size();i++){
			this.addRequire((Require)requirement.get(i));
		}
		
		Category category = new Category();
		category.setId(user.getGroupId());
		this.addCategory(category);

		return true;
	}
	/*
	 * method used to extract info from pom.xml
	 */
	public boolean construct(MavenProject project, ExtractBindexInfo ebi){
		
		if(ebi.getPresentationName() != null)
		{
			this.setPresentationName(ebi.getPresentationName());
			if(project.getName() != null)
				m_logger.warn("pom property override:<presentationname> "+project.getName());
		}
		else
			this.setPresentationName(project.getName());
		
		if(ebi.getSymbolicName() != null)
		{
			this.setSymbolicName(ebi.getSymbolicName());
			if(project.getArtifactId() != null)
				m_logger.warn("pom property override:<symbolicname> "+project.getArtifactId());
		}
		else
			this.setSymbolicName(project.getArtifactId());
	
		if(ebi.getVersion() != null)
		{
			this.setVersion(ebi.getVersion());
			if(project.getVersion() != null)
				m_logger.warn("pom property override:<version> "+project.getVersion());
		}
		else
			this.setVersion(project.getVersion());


		if(ebi.getDescription() != null)
		{
			this.setDescription(ebi.getDescription());
			if(project.getDescription() != null)
				m_logger.warn("pom property override:<description> "+project.getDescription());
		}
		else
			this.setDescription(project.getDescription());
		
		
		if(ebi.getDocumentation() != null)
		{
			this.setDocumentation(ebi.getDocumentation());
			if(project.getUrl() != null)
				m_logger.warn("pom property override:<documentation> "+project.getUrl());
		}
		else
			this.setDocumentation(project.getUrl());
		
		if(ebi.getSource() != null)
		{
			this.setSource(ebi.getSource());
			if(project.getScm() != null)
				m_logger.warn("pom property override:<source> "+project.getScm());
		}
		else
		{
			String src = null;
			if(project.getScm() != null)
				src = project.getScm().getUrl();
			this.setSource(src);
		}
		
		if(ebi.getLicense() != null)
		{
			this.setLicense(ebi.getLicense());
			String lic = null;
			List l = project.getLicenses();
			Iterator it = l.iterator();
			while(it.hasNext())
			{
				if(it.next() != null)
				{
					m_logger.warn("pom property override:<source> "+lic);
					break;
				}
			}
		}
		else
		{
			String lic = null;
			List l = project.getLicenses();
			Iterator it = l.iterator();
			while(it.hasNext())
				lic = it.next() + ";";
			
			this.setLicense(lic);
		}
		
//		create the first capability (ie : bundle)
		Capability capability = new Capability();
		capability.setName("bundle");
		PElement p = new PElement();
		p.setN("manifestversion");
		p.setV("2");
		capability.addP(p);
		
		p = new PElement();
		p.setN("presentationname");
		p.setV(this.getPresentationName());
		capability.addP(p);
		
		p = new PElement();
		p.setN("symbolicname");
		p.setV(this.getSymbolicName());
		capability.addP(p);
		
		p = new PElement();
		p.setN("version");
		p.setT("version");
		p.setV(this.getVersion());
		capability.addP(p);
		
		this.addCapability(capability);
		
		ArrayList capabilities = (ArrayList) ebi.getCapabilities();
		for(int i=0;i<capabilities.size();i++){
			this.addCapability((Capability)capabilities.get(i));
		}
		
		ArrayList requirement = (ArrayList) ebi.getRequirement();
		for(int i=0;i<requirement.size();i++){
			this.addRequire((Require)requirement.get(i));
		}
		
		//we also add the goupId
		Category category = new Category();
		category.setId(project.getGroupId());
		this.addCategory(category);
		
		return true;
	}

	public String getId() {
		return m_id;
	}

	public void setId(String m_id) {
		this.m_id = m_id;
	}
	
	public boolean isValid(){
		//we must verify require properties are present
		boolean result = this.getPresentationName() != null && this.getSymbolicName() != null &&
						 this.getVersion() != null && this.getUri() != null && this.getSize() != null;
		return result;
	}
	private String getInvalidProperties(){
		if(this.isValid()){
			if(this.getId() == null)
				return "id";
			else
				return "";
		}
		String result = "";
		if(this.getPresentationName() == null)
			result = result + "presentationName;";
		if(this.getSymbolicName() == null)
			result = result + "symbolicName;";
		if(this.getVersion() == null)
			result = result + "version;";
		if(this.getUri() == null)
			result = result + "Uri;";
		if(this.getSize() == null)
			result = result + "Size";
		return result;
	}
	
	public boolean isSameBundleResource(String symbolicName, String presentationName, String version){
		if(this.isValid()){
			boolean result;
			result = (symbolicName.compareTo(this.getSymbolicName())==0) && (version.compareTo(this.getVersion())==0) &&
					 (presentationName.compareTo(this.getPresentationName())==0);
			return result;
			
		}
		else{
			return false;
		}
			
	}
}
